;------------------------------------------------------------------------------
; NAME: Read_Dataheader_IR
;
; PURPOSE:
;    This routine read the header and the data from the .dat data file and store its content.
;    
; SVN revision information:
;     $Rev: 1614 $
;     $Author: chapron $
;     $Date: 2013-11-28 17:17:37 +0100 (jeu., 28 nov. 2013) $     
;
; CALLING SEQUENCE:
;    None
;   
; INPUTS: 
;    IR_DATA_pathname : Nom complet du fichier .dat  
;   
; OUTPUTS: 
;    time_start    : Temps de debut d observation 
;    time_end      : Temps de fin d observation
;    Hearder       : Informations invariantes tout au long de l observation
;                    extraite du premier header. (50 points)
;    INFOSPEC      :
;    INFOSERV      :
;    COMMAND       :
;    InfoServArray : 
;    TimeArray     :
;    RELATIVE_Time :
;    SpectrArr1    :
;    SpectrArr2    :
;    FreqArray     :
;    
; COMMON BLOCKS :
;    None 
;
; MODIFICATION HISTORY: 
; Nov  2013 - Nicolas Chapron, updated to read parameters and data, 
;                              part extracted from SPICAMIRCALIBRATION_NEWARCHIVE_AUTO_2012 (Anna Fedorova - IKI)
; Dec  2012 - Gaetan Lacombe, - add header
;                             - refactored code
;                             - old routine name : ReaddataUVtime.pro
;                             
; 
;##############################################################################

PRO spim_read_dataheader_IR, IR_DATA_pathname, time_start, time_end, INFOSPEC, INFOSERV, COMMAND, $
                             Header, InfoServArray, TimeArray, RELATIVE_Time, SpectrArr1, SpectrArr2, FreqArray


;------------------------------------------------------------------------------    
;---------------     INITIALISATION DES PARAMETRES      -----------------------

    COMMAND = { MODE     : { EXIT:0, SOURCE:0, DETS:0, TIME:0, SWAP_DETS : 0, GAIN:0 }              ,$
                DAC      : 0                                                                        ,$
                WINDOW_1 : {FREQUENCY : 0.0, POINTS : 0 ,STEP : 0.0, FRQ_OFFSET : 0,FRQ_RANGE : 0}  ,$
                WINDOW_2 : {FREQUENCY : 0.0, POINTS : 0, STEP : 0.0, FRQ_OFFSET : 0,FRQ_RANGE : 0}  ,$
                WINDOW_3 : {FREQUENCY : 0.0, POINTS : 0, STEP : 0.0, FRQ_OFFSET : 0,FRQ_RANGE : 0}  ,$
                CONFIG   : { COMMAND_DESCRIPTOR:0, DOTS_DESCRIPTOR:0 }                               $
              }

    SUTRP1Tempi  = 0.0
    SUTRP2Tempi  = 0.0
    SolarShuTemp = 0.0
    StructTemp   = 0.0
    LCLCurrenti  = 0.0
    
    INFOSERV = { TemperatureofSUTRP1      : 0.0,$
                 TemperatureofSUTRP2      : 0.0,$
                 TemperatureofsolarShutter: 0.0,$
                 TemperatureofStructure   : 0.0,$
                 Det0Temp                 : 0.0 ,$
                 Det1Temp                 : 0.0 ,$
                 AOTFTemp                 : 0.0 ,$
                 BaseTemp                 : 0.0 ,$
                 RFPower                  : 0.0 ,$
                 SuppVolt                 : 0.0 ,$
                 LCLCurrent               : 0.0 $
               }

    TimeSession = { year   : 0, $
                    month  : 0, $
                    day    : 0, $
                    hour   : 0, $
                    minute : 0, $
                    second : 0, $
                    msecond: 0d }
                    
    INFOSPEC = { Channels         : 0,$
                 ExpectedPoints   : 0,$
                 NumberofSpectra  : 0,$
                 NumberofSessions : 0 }                    

;------------------------------------------------------------------------------    
;---------------        LECTURE DU PREMIER HEADER       -----------------------
; Recuperation des informations invariantes tout au long de l observation dans
; le premier header.
    Header = intarr(50)
    OPENR, lun, IR_DATA_pathname, /GET_LUN
    readu, lun, Header

    COMMAND.MODE.EXIT                 = Header[18]
    COMMAND.MODE.SOURCE               = Header[19]
    COMMAND.MODE.DETS                 = Header[20]
    COMMAND.MODE.SWAP_DETS            = Header[21]
    COMMAND.MODE.TIME                 = Header[22]
    COMMAND.DAC                       = Header[23]
    COMMAND.MODE.GAIN                 = Header[25]
    COMMAND.WINDOW_1.FRQ_OFFSET       = Header[27]
    COMMAND.WINDOW_1.FRQ_RANGE        = Header[28]
    COMMAND.WINDOW_1.POINTS           = Header[29]
    COMMAND.WINDOW_1.STEP             = Header[30]
    COMMAND.WINDOW_2.FRQ_OFFSET       = Header[31]
    COMMAND.WINDOW_2.FRQ_RANGE        = Header[32]
    COMMAND.WINDOW_2.POINTS           = Header[33]
    COMMAND.WINDOW_2.STEP             = Header[34]
    COMMAND.WINDOW_3.FRQ_OFFSET       = Header[35]
    COMMAND.WINDOW_3.FRQ_RANGE        = Header[36]
    COMMAND.WINDOW_3.POINTS           = Header[37]
    COMMAND.WINDOW_3.STEP             = Header[38]
    COMMAND.CONFIG.COMMAND_DESCRIPTOR = Header[39]
    COMMAND.CONFIG.DOTS_DESCRIPTOR    = Header[40]
    COMMAND.WINDOW_1.FREQUENCY        = Header[45]
    COMMAND.WINDOW_2.FREQUENCY        = Header[46]
    COMMAND.WINDOW_3.FREQUENCY        = Header[47]
        
    INFOSPEC.expectedpoints   = Header[42]
    INFOSPEC.numberofspectra  = Header[43]
    INFOSPEC.Numberofsessions = Header[44]
    INFOSPEC.channels         = Header[20]

    FreqArray        = fltarr(INFOSPEC.expectedpoints)
    InfoServArray    = replicate(INFOSERV, INFOSPEC.numberofspectra)
    TimeArray        = replicate(TimeSession, INFOSPEC.numberofspectra)
    RELATIVE_Time    = LON64ARR(INFOSPEC.NumberofSpectra)
    SpectrArr1       = fltarr(INFOSPEC.expectedpoints, INFOSPEC.numberofspectra)
    SpectrArr2       = fltarr(INFOSPEC.expectedpoints, INFOSPEC.numberofspectra)
    spec1            = fltarr(INFOSPEC.expectedpoints)
    spec2            = fltarr(INFOSPEC.expectedpoints)
    
;------------------------------------------------------------------------------    
;--------     LECTURE DES RECORDS : RECUPERATION DES INFORMATIONS     ---------

    READU, lun, FreqArray
    FOR ii = 0, INFOSPEC.numberofspectra - 1 DO BEGIN
      
          READU, lun, TimeSession,INFOSERV
          
          IF INFOSPEC.channels EQ 1 THEN BEGIN
                readu, lun, spec1
          ENDIF ELSE BEGIN
                readu, lun, spec1
                readu, lun, spec2
          ENDELSE
          
          FOR j = 0, INFOSPEC.ExpectedPoints - 1 DO BEGIN
               IF (spec1(j) LT -1000) THEN spec1(j) = 4096 + spec1(j)
               IF (spec2(j) LT -1000) THEN spec2(j) = 4096 + spec2(j)
          ENDFOR

          InfoServArray[ii]    = INFOSERV
          TimeArray[ii]        = TimeSession
          SpectrArr1[*,ii]     = spec1
          SpectrArr2[*,ii]     = spec2
          
          RELATIVE_Time[ii]    = timeSession.month * 30. * 24. * 60. * 60. + $
                                 timeSession.day * 24. * 60. * 60. + $
                                 timeSession.hour * 60. * 60. + $
                                 timeSession.minute * 60. + $
                                 timeSession.second
          
          spec1 = fltarr(INFOSPEC.ExpectedPoints)
          spec2 = fltarr(INFOSPEC.ExpectedPoints)

    ENDFOR
    RELATIVE_Time = RELATIVE_Time - RELATIVE_Time[0] + 1


    CLOSE, lun
    FREE_LUN, lun


    time_start = strcompress(TimeArray[0].year) + '/' + $
                 strcompress(TimeArray[0].month) + '/' + $
                 strcompress(TimeArray[0].day) + '  ' + $
                 strcompress(TimeArray[0].hour) + ':' + $
                 strcompress(TimeArray[0].minute) + ':' + $
                 strcompress(TimeArray[0].second) + '.' + $
                 strtrim(fix(TimeArray[0].msecond),1)


    time_end = strcompress(TimeArray[INFOSPEC.numberofspectra - 1].year) + '/' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].month) + '/' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].day) + '  ' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].hour) + ':' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].minute) + ':' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].second) + '.' + $
               strtrim(fix(TimeArray[INFOSPEC.numberofspectra - 1].msecond), 1)



END