;------------------------------------------------------------------------------
; NAME: spiv_Read_Dataheader_IR
;
; PURPOSE:
;    This routine read the header from the .dat data file and store its content.
;    
; SVN revision information:
;     $Rev: 1050 $
;     $Author: glacombe $
;     $Date: 2013-01-22 18:07:18 +0100 (mar., 22 janv. 2013) $     
;
; CALLING SEQUENCE:
;    None
;   
; INPUTS: 
;    IR_DATA_pathname : .dat full path  
;   
; OUTPUTS: 
;    time_start, 
;    time_end, 
;    NumberofSpectra, 
;    NumberofSessions,
;    ExpectedPoints,
;    Channels,
;    COMMAND,
;    Header,
;    SpectrArr1,
;    SpectrArr2,
;    FreqArray
;    
; COMMON BLOCKS :
;    None 
;
; MODIFICATION HISTORY: 
; June 2014 - Nicolas Chapron, updated to read parameters and data, 
;                              part extracted from calibration_SPICAVIR_archive_pol_mars2011_alfa (Anna Fedorova - IKI)
; Dec  2012 - Gaetan Lacombe, - add header
;                             - refactored code
;                             - old routine name : spiv_readdatair.pro and read_data_ir
; Jan  2007 - A.REBERAC       - original routine updated for VEX SPICAV IR DATA 
;                               by C. NEHME
;                               This routine read all IR spectra from a binary 
;                               file the binary file contains :
;                               A general header , with 50 points 
;##############################################################################

PRO spiv_Read_Dataheader_IR, IR_DATA_pathname, time_start, time_end, INFOSPEC, NumberofSpectra, NumberofSessions, $
                             ExpectedPoints, Channels, COMMAND, Header, InfoServArray, TimeArray, RELATIVE_Time, SpectrArr1, SpectrArr2, FreqArray


;------------------------------------------------------------------------------    
;---------------     INITIALISATION DES PARAMETRES      -----------------------


    COMMAND = {MODE     : {EXIT : 0, SOURCE : 0, DETS : 0, SWAP_DETS : 0, TIME : 0}               , $
               DAC      : {DAC : 0, GAINBOOST : 0, GAIN : 0}                                      , $
               WINDOW_1 : {FREQUENCY : 0.0, POINTS : 0, STEP : 0.0, FRQ_OFFSET : 0,FRQ_RANGE : 0} , $
               WINDOW_2 : {FREQUENCY : 0.0, POINTS : 0, STEP : 0.0, FRQ_OFFSET : 0,FRQ_RANGE : 0} , $
               WINDOW_3 : {FREQUENCY : 0.0, POINTS : 0, STEP : 0.0 , FRQ_OFFSET : 0,FRQ_RANGE : 0}, $
               CONFIG   : {COMMAND_DESCRIPTOR : 0, DOTS_DESCRIPTOR : 0}                             }

    INFOSERV = {Det0Temp : 0.0 , $
                Det1Temp : 0.0 , $
                AOTFTemp : 0.0 , $
                BaseTemp : 0.0 , $
                RFPower  : 0.0 , $
                SuppVolt : 0.0   }

    TimeSession = {TIME2      , $
                   year    : 0, $
                   month   : 0, $
                   day     : 0, $
                   hour    : 0, $
                   minute  : 0, $
                   second  : 0, $
                   msecond : 0d }

    INFOSPEC = {Channels         : 0, $
                ExpectedPoints   : 0, $
                NumberofSpectra  : 0, $
                NumberofSessions : 0  }


;------------------------------------------------------------------------------    
;---------------        LECTURE DU PREMIER HEADER       -----------------------
; Recuperation des informations invariantes tout au long de l observation dans
; le premier header.
    Header = intarr(50)
    OPENR, LUNIN, IR_DATA_pathname, /GET_LUN
    READU, LUNIN, Header

    COMMAND.MODE.EXIT                 = Header[18]
    COMMAND.MODE.SOURCE               = Header[19]
    COMMAND.MODE.DETS                 = Header[20]
    COMMAND.MODE.SWAP_DETS            = Header[21]
    COMMAND.MODE.TIME                 = Header[22]
    COMMAND.DAC.DAC                   = Header[23]
    COMMAND.DAC.GAINBOOST             = Header[24]
    COMMAND.DAC.GAIN                  = Header[25]
    COMMAND.WINDOW_1.FRQ_OFFSET       = Header[27]
    COMMAND.WINDOW_1.FRQ_RANGE        = Header[28]
    COMMAND.WINDOW_1.POINTS           = Header[29]
    COMMAND.WINDOW_1.STEP             = Header[30]
    COMMAND.WINDOW_2.FRQ_OFFSET       = Header[31]
    COMMAND.WINDOW_2.FRQ_RANGE        = Header[32]
    COMMAND.WINDOW_2.POINTS           = Header[33]
    COMMAND.WINDOW_2.STEP             = Header[34]
    COMMAND.WINDOW_3.FRQ_OFFSET       = Header[35]
    COMMAND.WINDOW_3.FRQ_RANGE        = Header[36]
    COMMAND.WINDOW_3.POINTS           = Header[37]
    COMMAND.WINDOW_3.STEP             = Header[38]
    COMMAND.CONFIG.COMMAND_DESCRIPTOR = Header[39]
    COMMAND.CONFIG.DOTS_DESCRIPTOR    = Header[40]
    
    MODE_TIME = [1.4, 2.8, 5.6, 11.2, 22.4, 44.8, 89.6, 179.2]

    MODE_GAIN = [1, 2, 4, 8, 16, 32, 64, 128]
    
    
    INFOSPEC.expectedpoints   = Header[42]
    INFOSPEC.numberofspectra  = Header[43]
    INFOSPEC.Numberofsessions = Header[44]
    INFOSPEC.channels         = Header[20]

    FreqArray        = fltarr(INFOSPEC.expectedpoints)
    InfoServArray    = replicate(INFOSERV,INFOSPEC.numberofspectra)
    TimeArray        = replicate(TimeSession,INFOSPEC.numberofspectra)
    RELATIVE_Time    = LON64ARR(INFOSPEC.NumberofSpectra)
    SpectrArr1       = fltarr(INFOSPEC.expectedpoints,INFOSPEC.numberofspectra)
    SpectrArr2       = fltarr(INFOSPEC.expectedpoints,INFOSPEC.numberofspectra)
    spec1            = fltarr(INFOSPEC.expectedpoints)
    spec2            = fltarr(INFOSPEC.expectedpoints)
    SolarShutterTemp = fltarr(INFOSPEC.numberofspectra)
    StructureTemp    = fltarr(INFOSPEC.numberofspectra)
    SUTRP1Temp       = fltarr(INFOSPEC.numberofspectra)
    SUTRP2Temp       = fltarr(INFOSPEC.numberofspectra)
    LCLCurrent       = fltarr(INFOSPEC.numberofspectra)
    SUTRP1Tempi      = 0.0
    SUTRP2Tempi      = 0.0
    SolarShuTemp     = 0.0
    StructTemp       = 0.0
    LCLCurrenti      = 0.0
    
    DOTSS = [0,15,35,60,45,30,50,85]
    Ndots = DOTSS(COMMAND.CONFIG.DOTS_DESCRIPTOR)


;------------------------------------------------------------------------------    
;--------     LECTURE DES RECORDS : RECUPERATION DES INFORMATIONS     ---------

    READU, LUNIN, FreqArray

    FOR ii = 0, INFOSPEC.numberofspectra - 1 DO BEGIN

          READU, LUNIN, TimeSession
          READU, LUNIN, SUTRP1Tempi
          READU, LUNIN, SUTRP2Tempi
          READU, LUNIN, SolarShuTemp
          READU, LUNIN, StructTemp
          READU, LUNIN, INFOSERV
          READU, LUNIN, LCLCurrenti
          READU, LUNIN, spec1
          READU, LUNIN, spec2

;------------------------------------------------------------------------------
;---------         Reparation du bit de signe pour visualisation        -------

;          Npoint_theor = CMD_W0_POINTS + CMD_W1_POINTS + CMD_W2_POINTS + CMD_DOTS
;          IF ExpectedPoints LT Npoint_theor THEN stop
;          
;          IF CMD_W0_POINTS GT 1 THEN BEGIN
;                if spec1[0] lt -500 then spec1(0) = 4096 + spec1(0)
;                if spec2[0] lt -500 then spec2(0) = 4096 + spec2(0)
;                FOR jj = 1, CMD_W0_POINTS - 1 DO BEGIN               
;                      if spec1[jj] lt 0 and abs(spec1[jj - 1] - spec1[jj]) gt 2048 then spec1(jj) = 4096 + spec1(jj)
;                      if spec2[jj] lt 0 and abs(spec2[jj - 1] - spec2[jj]) gt 2048 then spec2(jj) = 4096 + spec2(jj)      
;                ENDFOR          
;          ENDIF
;          IF CMD_W1_POINTS GT 1 THEN BEGIN
;                CUMUL_points = CMD_W0_POINTS + CMD_W1_POINTS
;                if spec1[CMD_W0_POINTS] lt -500 then spec1(CMD_W0_POINTS) = 4096 + spec1(CMD_W0_POINTS)
;                if spec2[CMD_W0_POINTS] lt -500 then spec2(CMD_W0_POINTS) = 4096 + spec2(CMD_W0_POINTS)
;                FOR jj = CMD_W0_POINTS + 1, CUMUL_points - 1 DO BEGIN               
;                      if spec1[jj] lt 0 and abs(spec1[jj - 1] - spec1[jj]) gt 2048 then spec1(jj) = 4096 + spec1(jj)
;                      if spec2[jj] lt 0 and abs(spec2[jj - 1] - spec2[jj]) gt 2048 then spec2(jj) = 4096 + spec2(jj)      
;                ENDFOR          
;          ENDIF
;          IF CMD_W2_POINTS GT 1 THEN BEGIN
;                CUMUL_points = CMD_W0_POINTS + CMD_W1_POINTS
;                if spec1[CUMUL_points] lt -500 then spec1(CUMUL_points) = 4096 + spec1(CUMUL_points)
;                if spec2[CUMUL_points] lt -500 then spec2(CUMUL_points) = 4096 + spec2(CUMUL_points)
;                FOR jj = CUMUL_points + 1, CUMUL_points + CMD_W2_POINTS - 1 DO BEGIN               
;                      if spec1[jj] lt 0 and abs(spec1[jj - 1] - spec1[jj]) gt 2048 then spec1(jj) = 4096 + spec1(jj)
;                      if spec2[jj] lt 0 and abs(spec2[jj - 1] - spec2[jj]) gt 2048 then spec2(jj) = 4096 + spec2(jj)      
;                ENDFOR          
;          ENDIF
          
;       Ancienne methode pour reparation de bit de signe          
;        for j=1,ExpectedPoints-1 do begin
;              if (spec1[j] lt -100) then spec1(j)=4096+spec1(j)
;              if (spec2[j] lt -100) then spec2(j)=4096+spec2(j)
;              if (spec1[j-1]-spec1[j] gt 3000) then spec1(j)=4096+spec1(j)
;              if (spec2[j-1]-spec2[j] gt 3000) then spec2(j)=4096+spec2(j)
;        endfor
;------------------------------------------------------------------------------

          ;--- Methode IKI - Extracted from calibration_SPICAVIR_archive_pol_mars2011_alfa (IKI, 2014)
           
          if (COMMAND.MODE.TIME lt 4) then begin
            for j=1,INFOSPEC.ExpectedPoints-1 do begin
              if (j lt INFOSPEC.ExpectedPoints-ndots) then begin
                if ( spec1[j-1]-spec1[j] gt 3500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec1(j)=4096+spec1(j)
                if ( spec2[j-1]-spec2[j] gt 3500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec2(j)=4096+spec2(j)
                if (freqarray[j] gt 140000d and spec1[j-1]-spec1[j] gt 2500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec1(j)=4096+spec1(j)
                if (freqarray[j] gt 140000d and spec2[j-1]-spec2[j] gt 2500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec2(j)=4096+spec2(j)
              endif
              if (COMMAND.CONFIG.DOTS_DESCRIPTOR gt 0 and j ge INFOSPEC.ExpectedPoints-ndots) then begin
                if (freqarray[j] gt 210000d) then ku=j
                if(freqarray[j] gt 210000d and spec1[j] lt -100) then spec1(j)=4096+spec1(j)
                if(freqarray[j] gt 210000d and spec2[j] lt -100) then spec2(j)=4096+spec2(j)
                if (freqarray[j] gt 200000d and freqarray[j] lt 210000d and spec1[ku] gt 100 and spec1[ku] gt spec1[j]) then spec1[j]=spec1[j]+4096
                if (freqarray[j] gt 200000d and freqarray[j] lt 210000d and spec2[ku] gt 100 and spec2[ku] gt spec2[j]) then spec2[j]=spec2[j]+4096
                ;          if (freqarray[j] gt 200000d and freqarray[j] lt 210000d and spec1[ku] gt 100 and spec1[j] le 0) then spec1[j]=spec1[j]+4096
                ;          if (freqarray[j] gt 200000d and freqarray[j] lt 210000d and spec2[ku] gt 100 and spec2[j] le 0) then spec2[j]=spec2[j]+4096
                if (freqarray[j] gt 200000d and freqarray[j] lt 210000d) then ku3=j
                if (freqarray[j] gt 170000d and freqarray[j] lt 180000d) then begin
                  if(spec1[ku3] gt spec1[j] and spec1[ku3] gt 30) then spec1(j)=4096+spec1(j)
                  if(spec2[ku3] gt spec2[j] and spec2[ku3] gt 30) then spec2(j)=4096+spec2(j)
                  if(spec1[ku3] gt spec1[j] and spec1[ku3] gt 30) then spec1(j)=4096+spec1(j)
                  if(spec2[ku3] gt spec2[j] and spec1[ku3] gt 30) then spec2(j)=4096+spec2(j)
                  if(spec1[ku3] gt spec1[j]/1.3 and spec1[ku3] gt 30) then spec1(j)=4096+spec1(j)
                  if(spec2[ku3] gt spec2[j]/1.3 and spec1[ku3] gt 30) then spec2(j)=4096+spec2(j)
                endif
                if (freqarray[j] gt 170000d and freqarray[j] lt 180000d) then ku2=j
                if (freqarray[j] lt 170000d and freqarray[j] gt 145000d) then begin
                  if (spec1[ku2] le 2047 and spec1[ku2]-spec1[j] gt 1200) then spec1(j)=4096+spec1(j)
                  if (spec2[ku2] le 2047 and spec2[ku2]-spec2[j] gt 1200) then spec2(j)=4096+spec2(j)
                  if (spec1[ku2] gt 2047) then begin
                    if (spec1[ku2]-spec1[j] ge 3000) then spec1(j)=4096+spec1(j)
                  endif
                  if (spec2[ku2] gt 2047) then begin
                    if (spec2[ku2]-spec2[j] ge 3000) then spec2(j)=4096+spec2(j)
                  endif
                  if (spec2[ku2]-spec2[j] gt 2000) then spec2(j)=4096+spec2(j)
                  if (spec1[ku2]-spec1[j] gt 2000) then spec1(j)=4096+spec1(j)
                endif
              endif
            endfor
            for j=1,INFOSPEC.ExpectedPoints-1 do begin
              if (freqarray[j] gt 140000d and spec1[j-1]-spec1[j] gt 1500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec1(j)=4096+spec1(j)
              if (freqarray[j] gt 140000d and spec2[j-1]-spec2[j] gt 1500 and j lt INFOSPEC.ExpectedPoints-ndots) then spec2(j)=4096+spec2(j)
            endfor
          endif

          ;------------------------------------------------------------------------------




          InfoServArray[ii]    = INFOSERV
          TimeArray[ii]        = TimeSession
          SpectrArr1[*,ii]     = spec1
          SpectrArr2[*,ii]     = spec2
      		SolarShutterTemp[ii] = SolarShuTemp
      		StructureTemp[ii]    = StructTemp
      		LCLCurrent[ii]       = LCLCurrenti
      		SUTRP1Temp[ii]       = SUTRP1Tempi
      		SUTRP2Temp[ii]       = SUTRP2Tempi
      		
          RELATIVE_Time[ii]    = timeSession.month * 30. * 24. * 60. * 60. + $
                                 timeSession.day * 24. * 60. * 60. + $
                                 timeSession.hour * 60. * 60. + $
                                 timeSession.minute * 60. + $
                                 timeSession.second
                               
    ENDFOR
    RELATIVE_Time = RELATIVE_Time - RELATIVE_Time[0] + 1
    
    CLOSE, LUNIN
    FREE_LUN, LUNIN



    time_start = strcompress(TimeArray[0].year) + '/' +   $
                 strcompress(TimeArray[0].month) + '/' +  $
                 strcompress(TimeArray[0].day) + '  ' +   $
                 strcompress(TimeArray[0].hour) + ':' +   $
                 strcompress(TimeArray[0].minute) + ':' + $
                 strcompress(TimeArray[0].second) + '.' + $
                 strtrim(fix(TimeArray[0].msecond), 1)


    time_end = strcompress(TimeArray[INFOSPEC.numberofspectra - 1].year) + '/' +   $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].month) + '/' +  $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].day) + '  ' +   $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].hour) + ':' +   $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].minute) + ':' + $
               strcompress(TimeArray[INFOSPEC.numberofspectra - 1].second) + '.' + $
               strtrim(fix(TimeArray[INFOSPEC.numberofspectra - 1].msecond), 1)



END